package response

import (
	"eta/eta_chart_lib/models"
	"eta/eta_chart_lib/models/request"
	"eta/eta_chart_lib/services/excel"
	"time"
)

// ExcelTableDetailResp  excel表格详情
type ExcelTableDetailResp struct {
	//ChartInfo   *ChartInfo
	UniqueCode    string `description:"表格唯一code"`
	Source        int    `description:"表格来源，1：excel插件的表格，2：自定义表格，默认：1"`
	ExcelType     int    `description:"表格类型，1：指标列，2：日期列，默认：1"`
	ExcelImage    string `description:"表格截图"`
	ExcelName     string `description:"表格名称"`
	SourcesFrom   string `description:"图表来源"`
	ExcelSource   string `description:"表格来源str"`
	ExcelSourceEn string `description:"表格来源（英文）"`
	TableInfo     excel.TableData
	Config        ExcelTableDetailConfigResp
}

// ExcelTableDetailConfigResp
// @Description: Excel表格的配置信息
type ExcelTableDetailConfigResp struct {
	FontSize int
}

// TableCellResp 单元格
type TableCellResp struct {
	DataType  int    `description:"数据类型，1：普通的，2：插值法，3：手动输入，4：公式计算"`
	DataTime  string `description:"所属日期"`
	ShowValue string `description:"展示的值"`
	Value     string `description:"实际值（计算公式）"`
}

type TableDataItem struct {
	EdbInfoId int                     `description:"指标id"`
	Data      []request.ManualDataReq `description:"数据列表"`
}

// TableDetailResp  excel表格详情
type TableDetailResp struct {
	ExcelInfo models.ExcelInfo     `description:"表格基础信息"`
	TableData request.TableDataReq `description:"表格内容"`
}

// ExcelInfoDetail excel表格详情(前端使用)
type ExcelInfoDetail struct {
	ExcelInfoId     int                  `orm:"column(excel_info_id);pk"`
	Source          int                  `description:"表格来源，1：excel插件的表格，2：自定义表格，默认：1"`
	ExcelType       int                  `description:"表格类型，1：指标列，2：日期列，默认：1"`
	ExcelName       string               `description:"表格名称"`
	UniqueCode      string               `description:"表格唯一编码"`
	ExcelClassifyId int                  `description:"表格分类id"`
	SysUserId       int                  `description:"操作人id"`
	SysUserRealName string               `description:"操作人真实姓名"`
	Content         string               `description:"表格内容"`
	ExcelImage      string               `description:"表格图片"`
	FileUrl         string               `description:"表格下载地址"`
	Sort            int                  `description:"排序字段，数字越小越排前面"`
	IsDelete        int                  `description:"是否删除，0：未删除，1：已删除"`
	ModifyTime      time.Time            `description:"最近修改日期"`
	CreateTime      time.Time            `description:"创建日期"`
	TableData       request.TableDataReq `description:"表格内容"`
}
